<footer>
  {% if (theme_prev_next_buttons_location == 'bottom' or theme_prev_next_buttons_location == 'both') and (next or prev) %}
  <div class="rst-footer-buttons" role="navigation" aria-label="footer navigation">
    {% if next %}
    <a href="{{ next.link|e }}" class="btn btn-neutral float-right" title="{{ next.title|striptags|e }}" accesskey="n"
      rel="next">{{ _('Next') }} <span class="fa fa-arrow-circle-right"></span></a>
    {% endif %}
    {% if prev %}
    <a href="{{ prev.link|e }}" class="btn btn-neutral float-left" title="{{ prev.title|striptags|e }}" accesskey="p"
      rel="prev"><span class="fa fa-arrow-circle-left"></span> {{ _('Previous') }}</a>
    {% endif %}
  </div>
  {% endif %}

  <hr />

  <div style="display:flex; justify-content: space-between;">
    <div role="contentinfo">
      <p>
        {%- if show_copyright %}
        {%- if hasdoc('copyright') %}
        {% set path = pathto('copyright') %}
        {% set copyright = copyright|e %}
        &copy; <a href="{{ path }}">{% trans %}Copyright{% endtrans %}</a> {{ copyright }}
        {%- else %}
        {% set copyright = copyright|e %}
        &copy; {% trans %}Copyright{% endtrans %} {{ copyright }}
        {%- endif %}
        {%- endif %}

        {%- if build_id and build_url %}
        <span class="build">
          {# Translators: Build is a noun, not a verb #}
          {% trans %}Build{% endtrans %}
          <a href="{{ build_url }}">{{ build_id }}</a>.
        </span>
        {%- elif commit %}
        <span class="commit">
          {% trans %}Revision{% endtrans %} <code>{{ commit }}</code>.
        </span>
        {%- elif last_updated %}
        <span class="lastupdated">
          {% trans last_updated=last_updated|e %}Last updated on {{ last_updated }}.{% endtrans %}
        </span>
        {%- endif %}

      </p>
    </div>
    <div>
      <a href="https://www.netlify.com">
        <img src="https://www.netlify.com/img/global/badges/netlify-light.svg" />
      </a>
    </div>
  </div>

</footer>
