# Copyright (C) 2019 Cancer Care Associates

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#     http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from pymedphys._imports import numpy as np


def transform_axis(x, y, transform):
    transformed_x = transform @ np.vstack([x, np.zeros(len(x)), np.ones(len(x))])
    transformed_y = transform @ np.vstack([np.zeros(len(y)), y, np.ones(len(y))])

    return transformed_x[0:2], transformed_y[0:2]
