from pymedphys._base.delivery import DeliveryBase
from pymedphys._dicom.delivery import DeliveryDicom
from pymedphys._icom.delivery import DeliveryIcom
from pymedphys._monaco.delivery import DeliveryMonaco
from pymedphys._mosaiq.delivery import DeliveryMosaiq
from pymedphys._metersetmap.delivery import DeliveryMetersetMap
from pymedphys._trf.decode.delivery import DeliveryLogfile


class Delivery(  # type: ignore
    DeliveryMosaiq,
    DeliveryMetersetMap,
    DeliveryLogfile,
    DeliveryDicom,
    DeliveryMonaco,
    DeliveryIcom,
    DeliveryBase,
):
    pass
